/*
 * This program is free software; you can redistribute it and/or
 * modify it under the terms of the GNU General Public License
 * as published by the Free Software Foundation; either version 2
 * of the License, or (at your option) any later version.
 *
 * This program is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 *
 * You should have received a copy of the GNU General Public License
 * along with this program; if not, write to the Free Software
 * Foundation, Inc., 51 Franklin Street, Fifth Floor, Boston, MA  02110-1301, USA.
 * 
 * Contact Information:
 * Dambach-Werke GmbH
 * Elektronische Leitsysteme
 * Fritz-Minhardt-Str. 1
 * 76456 Kuppenheim
 * Phone: +49-7222-402-0
 * Fax: +49-7222-402-200
 * mailto: info@els.dambach.de
 */

package de.bsvrz.ste.rdstmc.junit.prueffall2;

import static org.junit.Assert.assertNotNull;
import static org.junit.Assert.assertTrue;

import org.junit.After;
import org.junit.AfterClass;
import org.junit.Before;
import org.junit.BeforeClass;
import org.junit.Test;

import de.bsvrz.dav.daf.main.ResultData;
import de.bsvrz.dav.daf.main.config.SystemObject;
import de.bsvrz.ste.rdstmc.allgemein.Beschleunigung;
import de.bsvrz.ste.rdstmc.junit.RDSTMCPrueffall;
import de.bsvrz.ste.rdstmc.junit.tools.JUnitHelper;
import de.bsvrz.ste.rdstmc.junit.tools.WarteAufDaten;
import de.bsvrz.ste.rdstmc.meldungserstellung.VerwaltungMeldungen;
import de.bsvrz.ste.rdstmc.test.Prueffall2;
import de.bsvrz.ste.rdstmc.test.PrueffallUmgebung;
import de.bsvrz.sys.funclib.debug.Debug;

/**
 * @author Sans
 *
 */
public class Prueffall2_1 extends RDSTMCPrueffall
{
  /**
   * Debug-Logger fr Logging-Ausgaben
   */
  private final static Debug _debug = Debug.getLogger();
  
  private static long _testGeneratorStart;

  @BeforeClass
  public static void setUpBeforeClass() throws Exception 
  {
    Debug.setLoggerLevel("DEFAULT", Debug.CONFIG);
    Debug.setHandlerLevel("StdErr", Debug.CONFIG);
            
    Thread testGenerator = new Thread(new Prueffall2(JUnitHelper.getInstance().getConnection(), true));
    testGenerator.start();
    setTestGenerator(testGenerator);
    _testGeneratorStart = getConnection().getTime();
  }

  @AfterClass
  public static void tearDownAfterClass() throws Exception
  {
  }

  @Before
  public void setUp() throws Exception 
  {
  }

  @After
  public void tearDown() throws Exception
  {
  }
      
  @Test
  public void testPrueffall_2_1()throws Exception
  {
    assertTestGeneratorLaeuft();

    SystemObject meldung4 = getDataModel().getObject(VerwaltungMeldungen.PID_PREFIX + "4");
    assertNotNull("Objekt " + VerwaltungMeldungen.PID_PREFIX + "4" + " existiert", meldung4);
    SystemObject meldung5 = getDataModel().getObject(VerwaltungMeldungen.PID_PREFIX + "5");
    assertNotNull("Objekt " + VerwaltungMeldungen.PID_PREFIX + "5" + " existiert", meldung5);
    
    WarteAufDaten wad = new WarteAufDaten(getConnection(), 
        new SystemObject[] { meldung4, meldung5 }, getDataDescription(), 
        PrueffallUmgebung.APP_START_VERZOEGERUNG);

    long aktuelleZeit = getConnection().getTime();
    long rest = PrueffallUmgebung.APP_START_VERZOEGERUNG - (aktuelleZeit - _testGeneratorStart);
    assertTrue("Zu prfende Applikation liefert Daten innerhalb von " + 
        (PrueffallUmgebung.APP_START_VERZOEGERUNG / 1000) + "s", rest > 0);
    setStartZeitpunkt(aktuelleZeit + rest + KORREKTUR - 720000 / Beschleunigung.WERT);
    
    ResultData rd4 = wad.getDaten().get(meldung4.getPid());
    ResultData rd5 = wad.getDaten().get(meldung5.getPid());
    
    assertTrue("Daten fr Objekt " + meldung4.getPid() + " empfangen",
        rd4 != null && rd4.getObject().equals(meldung4)); 
    assertTrue("Daten fr Objekt " + meldung5.getPid() + " empfangen",
        rd5 != null && rd5.getObject().equals(meldung5));
    
    assertMeldungsDatenKorrekt(rd4,
        InformationsTyp.ALERTC,
        MeldungsArt.AUFHEBUNG,
        GeografischeBedeutung.NATIONAL,
        ZeitlicheDringlichkeit.NORMAL,
        ZeitlicheDauer.UNBEKANNT,
        -1,
        108,
        11907,
        null,
        null);
    
    assertMeldungsDatenKorrekt(rd5,
        InformationsTyp.ALERTC,
        MeldungsArt.AKTUALISIERUNG,
        GeografischeBedeutung.NATIONAL,
        ZeitlicheDringlichkeit.NORMAL,
        ZeitlicheDauer.UNBEKANNT,
        -1,
        131,
        11906,
        null,
        null);
  }

}
