/*
 * This program is free software; you can redistribute it and/or
 * modify it under the terms of the GNU General Public License
 * as published by the Free Software Foundation; either version 2
 * of the License, or (at your option) any later version.
 *
 * This program is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 *
 * You should have received a copy of the GNU General Public License
 * along with this program; if not, write to the Free Software
 * Foundation, Inc., 51 Franklin Street, Fifth Floor, Boston, MA  02110-1301, USA.
 * 
 * Contact Information:
 * Dambach-Werke GmbH
 * Elektronische Leitsysteme
 * Fritz-Minhardt-Str. 1
 * 76456 Kuppenheim
 * Phone: +49-7222-402-0
 * Fax: +49-7222-402-200
 * mailto: info@els.dambach.de
 */

package de.bsvrz.ste.rdstmc.konfiguration;

import java.util.Collections;
import java.util.Comparator;
import java.util.LinkedHashSet;
import java.util.Set;
import java.util.SortedSet;
import java.util.TreeSet;

import de.bsvrz.dav.daf.main.config.SystemObject;
import de.bsvrz.sys.funclib.debug.Debug;

/**
 * Es wird ein Strassenteilsegment modelliert.
 * 
 * @author Dambach Werke GmbH
 * @version $Revision: 1.1 $ / $Date: 2008/10/31 13:49:25 $ / ($Author: Sans $)
 */
public class StrassenTeilSegment extends DavProxyObjekt 
{
  /**
   * Debug-Logger fr Logging-Ausgaben
   */
  private final static Debug _debug = Debug.getLogger();
  
  /**
   * Lnge des Straenteilsegments
   */
  private long _laenge = 0;
  
  /**
   * Anzahl Fahrstreifen
   */
  private int _anzahlFahrStreifen = 0;
  
  /**
   * Steigung/Geflle
   */
  private String _steigungGefaelle = null;
  
  /**
   * Offset innerhalb des Straensegments
   */
  private long _offset = 0;
  
  
  /**
   * Erstellt eine neue Instanz vom Typ <code>StrassenSegment</code>
   * 
   * @param davObjekt das zuzuordnente Dav-Objekt
   * @param laenge die Lnge in m des Strassensegments
   */
  public StrassenTeilSegment(SystemObject davObjekt, long laenge, int anzahlFahrStreifen,
      String steigungGefaelle, long offset)
  {
    super(davObjekt);
    _laenge = laenge;
    _anzahlFahrStreifen = anzahlFahrStreifen;
    _steigungGefaelle = steigungGefaelle;
    _offset = offset;
  }
  
  /**
   * @return die Lnge in m dieses Strassenteilsegments
   */
  public long getLaenge()
  {
    return _laenge; 
  }
  
  /**
   * @return die Anzahl der Fahrstreifen dieses Strassenteilsegments
   */
  public int getAnzahlFahrStreifen()
  {
    return _anzahlFahrStreifen; 
  }
  
  /**
   * @return Steigung oder Geflle dieses Strassenteilsegments
   */
  public String getSteigungGefaelle()
  {
    return _steigungGefaelle; 
  }
  
  /**
   * @return den Offset dieses Strassenteilsegments auf seinem Strassensegment
   */
  public long getOffset()
  {
    return _offset; 
  }
  
}
