/*
 * This program is free software; you can redistribute it and/or
 * modify it under the terms of the GNU General Public License
 * as published by the Free Software Foundation; either version 2
 * of the License, or (at your option) any later version.
 *
 * This program is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 *
 * You should have received a copy of the GNU General Public License
 * along with this program; if not, write to the Free Software
 * Foundation, Inc., 51 Franklin Street, Fifth Floor, Boston, MA  02110-1301, USA.
 * 
 * Contact Information:
 * Dambach-Werke GmbH
 * Elektronische Leitsysteme
 * Fritz-Minhardt-Str. 1
 * 76456 Kuppenheim
 * Phone: +49-7222-402-0
 * Fax: +49-7222-402-200
 * mailto: info@els.dambach.de
 */

package de.bsvrz.ste.rdstmc.meldungserstellung;

import de.bsvrz.dav.daf.main.config.ConfigurationArea;

/**
 * Schnittstelle fr Datenelemente, die fr die Erstellung unf Verwaltung von
 * Meldungen global sind.
 * Implementierung z. B. ber Kommandozeilenargumente
 * 
 * @author Dambach Werke
 * @version $Revision: 1.1 $ / $Date: 2009/01/27 13:54:22 $ / ($Author: Sans $)
 */
public interface MeldungenGlobal 
{
  /**
   * @return den Konfigurationsbereich, in dem dynamische Meldungsobjekte abgelegt werden
   */
  public ConfigurationArea getKonfigurationsbereich();
  
  /**
   * @return die LocationMethode, nach der Meldungsdatenstze kodiert werden
   */
  public LocationMethode getLocationMethode();
  
  /**
   * @return die Methode, nach der beim Programmstart aktuelle Zustnde publiziert werden
   */
  public PublikationInitial getPublikationInitial();
  
  /**
   * @return die (Text-) Id der Organisationseinheit, die einer generierten Meldung zugeordnet sein soll
   * - i. d. R. LMS
   */
  public String getIdOrganisation();
  
  /**
   * @return die (numerische) Id der Organisationseinheit, die einer generierten Meldung zugeordnet sein soll
   * - i. d. R. LMS
   */
  public String getIdNummer();
  
  /**
   * @return Zeitspanne in min., wielange eine Meldung gltig (nicht abgelaufen)
   * sein soll, nachdem sie aktualisiert wurde
   */  
  public int getAblaufOffset();
  
}
