/*
 * This program is free software; you can redistribute it and/or
 * modify it under the terms of the GNU General Public License
 * as published by the Free Software Foundation; either version 2
 * of the License, or (at your option) any later version.
 *
 * This program is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 *
 * You should have received a copy of the GNU General Public License
 * along with this program; if not, write to the Free Software
 * Foundation, Inc., 51 Franklin Street, Fifth Floor, Boston, MA  02110-1301, USA.
 * 
 * Contact Information:
 * Dambach-Werke GmbH
 * Elektronische Leitsysteme
 * Fritz-Minhardt-Str. 1
 * 76456 Kuppenheim
 * Phone: +49-7222-402-0
 * Fax: +49-7222-402-200
 * mailto: info@els.dambach.de
 */

package de.bsvrz.ste.rdstmc.meldungserstellung;

import java.util.Collection;
import java.util.HashMap;
import java.util.Map;

import de.bsvrz.ste.rdstmc.allgemein.RDSEreignis;
import de.bsvrz.sys.funclib.debug.Debug;

public class RDSBeschreibung
{
  /**
   * Debug-Logger fr Logging-Ausgaben
   */
  private final static Debug _debug = Debug.getLogger();
  
  private static Map<RDSEreignis, Map<String, RDSBeschreibung>> DATEN_NORMAL = 
    new HashMap<RDSEreignis, Map<String, RDSBeschreibung>>();
  
  private static Map<RDSEreignis, Map<String, RDSBeschreibung>> DATEN_BAUSTELLE = 
    new HashMap<RDSEreignis, Map<String, RDSBeschreibung>>();
  
  public final static int KEINE_LAENGE = -1; 
  
  static
  {
    for (RDSEreignis ereignis: RDSEreignis.values())
    {
      DATEN_NORMAL.put(ereignis, new HashMap<String, RDSBeschreibung>());
      DATEN_BAUSTELLE.put(ereignis, new HashMap<String, RDSBeschreibung>());
    }
    
    DATEN_NORMAL.get(RDSEreignis.STAU).put("101", new RDSBeschreibung(KEINE_LAENGE, "(L) Stau"));    
    DATEN_NORMAL.get(RDSEreignis.STAU).put("102", new RDSBeschreibung(           1, "1 km Stau"));
    DATEN_NORMAL.get(RDSEreignis.STAU).put("103", new RDSBeschreibung(           2, "2 km Stau"));
    DATEN_NORMAL.get(RDSEreignis.STAU).put("129", new RDSBeschreibung(           3, "3 km Stau"));
    DATEN_NORMAL.get(RDSEreignis.STAU).put("104", new RDSBeschreibung(           4, "4 km Stau"));
    DATEN_NORMAL.get(RDSEreignis.STAU).put("105", new RDSBeschreibung(           6, "6 km Stau"));
    DATEN_NORMAL.get(RDSEreignis.STAU).put("106", new RDSBeschreibung(          10, "10 km Stau"));
    
    DATEN_BAUSTELLE.get(RDSEreignis.STAU).put("710", new RDSBeschreibung(KEINE_LAENGE, "Baustelle, (L) Stau"));
    DATEN_BAUSTELLE.get(RDSEreignis.STAU).put("711", new RDSBeschreibung(           1, "Baustelle, 1 km Stau"));
    DATEN_BAUSTELLE.get(RDSEreignis.STAU).put("712", new RDSBeschreibung(           2, "Baustelle, 2 km Stau"));
    DATEN_BAUSTELLE.get(RDSEreignis.STAU).put("812", new RDSBeschreibung(           3, "Baustelle, 3 km Stau"));
    DATEN_BAUSTELLE.get(RDSEreignis.STAU).put("713", new RDSBeschreibung(           4, "Baustelle, 4 km Stau"));
    DATEN_BAUSTELLE.get(RDSEreignis.STAU).put("714", new RDSBeschreibung(           6, "Baustelle, 6 km Stau"));
    DATEN_BAUSTELLE.get(RDSEreignis.STAU).put("715", new RDSBeschreibung(          10, "Baustelle, 10 km Stau"));
 
    DATEN_NORMAL.get(RDSEreignis.STOCKENDER_VERKEHR).put("108", new RDSBeschreibung(KEINE_LAENGE, "(L) stockender Verkehr"));
    DATEN_NORMAL.get(RDSEreignis.STOCKENDER_VERKEHR).put("109", new RDSBeschreibung(           1, "1 km stockender Verkehr"));
    DATEN_NORMAL.get(RDSEreignis.STOCKENDER_VERKEHR).put("110", new RDSBeschreibung(           2, "2 km stockender Verkehr"));
    DATEN_NORMAL.get(RDSEreignis.STOCKENDER_VERKEHR).put("131", new RDSBeschreibung(           3, "3 km stockender Verkehr"));
    DATEN_NORMAL.get(RDSEreignis.STOCKENDER_VERKEHR).put("111", new RDSBeschreibung(           4, "4 km stockender Verkehr"));
    DATEN_NORMAL.get(RDSEreignis.STOCKENDER_VERKEHR).put("112", new RDSBeschreibung(           6, "6 km stockender Verkehr"));
    DATEN_NORMAL.get(RDSEreignis.STOCKENDER_VERKEHR).put("113", new RDSBeschreibung(          10, "10 km stockender Verkehr"));
    
    DATEN_BAUSTELLE.get(RDSEreignis.STOCKENDER_VERKEHR).put("717", new RDSBeschreibung(KEINE_LAENGE, "Baustelle, (L) stockender Verkehr"));
    DATEN_BAUSTELLE.get(RDSEreignis.STOCKENDER_VERKEHR).put("718", new RDSBeschreibung(           1, "Baustelle, 1 km stockender Verkehr"));
    DATEN_BAUSTELLE.get(RDSEreignis.STOCKENDER_VERKEHR).put("719", new RDSBeschreibung(           2, "Baustelle, 2 km stockender Verkehr"));
    DATEN_BAUSTELLE.get(RDSEreignis.STOCKENDER_VERKEHR).put("813", new RDSBeschreibung(           3, "Baustelle, 3 km stockender Verkehr"));
    DATEN_BAUSTELLE.get(RDSEreignis.STOCKENDER_VERKEHR).put("720", new RDSBeschreibung(           4, "Baustelle, 4 km stockender Verkehr"));
    DATEN_BAUSTELLE.get(RDSEreignis.STOCKENDER_VERKEHR).put("721", new RDSBeschreibung(           6, "Baustelle, 6 km stockender Verkehr"));
    DATEN_BAUSTELLE.get(RDSEreignis.STOCKENDER_VERKEHR).put("722", new RDSBeschreibung(          10, "Baustelle, 10 km stockender Verkehr"));
    
    DATEN_NORMAL.get(RDSEreignis.DICHTER_VERKEHR).put("115", new RDSBeschreibung(KEINE_LAENGE, "(L) dichter Verkehr"));
    DATEN_NORMAL.get(RDSEreignis.DICHTER_VERKEHR).put("116", new RDSBeschreibung(           1, "1 km dichter Verkehr"));
    DATEN_NORMAL.get(RDSEreignis.DICHTER_VERKEHR).put("117", new RDSBeschreibung(           2, "2 km dichter Verkehr"));
    DATEN_NORMAL.get(RDSEreignis.DICHTER_VERKEHR).put("134", new RDSBeschreibung(           3, "3 km dichter Verkehr"));
    DATEN_NORMAL.get(RDSEreignis.DICHTER_VERKEHR).put("118", new RDSBeschreibung(           4, "4 km dichter Verkehr"));
    DATEN_NORMAL.get(RDSEreignis.DICHTER_VERKEHR).put("119", new RDSBeschreibung(           6, "6 km dichter Verkehr"));
    DATEN_NORMAL.get(RDSEreignis.DICHTER_VERKEHR).put("120", new RDSBeschreibung(          10, "10 km dichter Verkehr"));
    
    DATEN_BAUSTELLE.get(RDSEreignis.DICHTER_VERKEHR).put("724", new RDSBeschreibung(KEINE_LAENGE, "Baustelle, (L) dichter Verkehr"));
    DATEN_BAUSTELLE.get(RDSEreignis.DICHTER_VERKEHR).put("725", new RDSBeschreibung(           1, "Baustelle, 1 km dichter Verkehr"));
    DATEN_BAUSTELLE.get(RDSEreignis.DICHTER_VERKEHR).put("726", new RDSBeschreibung(           2, "Baustelle, 2 km dichter Verkehr"));
    DATEN_BAUSTELLE.get(RDSEreignis.DICHTER_VERKEHR).put("814", new RDSBeschreibung(           3, "Baustelle, 3 km dichter Verkehr"));
    DATEN_BAUSTELLE.get(RDSEreignis.DICHTER_VERKEHR).put("727", new RDSBeschreibung(           4, "Baustelle, 4 km dichter Verkehr"));
    DATEN_BAUSTELLE.get(RDSEreignis.DICHTER_VERKEHR).put("728", new RDSBeschreibung(           6, "Baustelle, 6 km dichter Verkehr"));
    DATEN_BAUSTELLE.get(RDSEreignis.DICHTER_VERKEHR).put("729", new RDSBeschreibung(          10, "Baustelle, 10 km dichter Verkehr"));
  }
  
//  public static String getCodeZuLaenge(RDSEreignis ereignis, long laenge, boolean baustelle)
//  {
//    Map<String, RDSBeschreibungAlt> resultMap = baustelle ? 
//        DATEN_BAUSTELLE.get(ereignis) : DATEN_NORMAL.get(ereignis);
// 
//    String codeOhneLaenge = null;    
//    String minCode = null;
//    String maxCode = null;
//    long min = 0;
//    long max = -1;    
//        
//    for (String code: resultMap.keySet())
//    {
//      RDSBeschreibungAlt obj = resultMap.get(code);
//      
//      if (obj._laenge < 0)
//      {
//        codeOhneLaenge = code;
//      }
//      else if (laenge >= 0)
//      {
//        if (obj._laenge <= laenge && obj._laenge > min)
//        {
//          min = obj._laenge;
//          minCode = code;
//        }
//        if (obj._laenge >= laenge && (max < 0 || obj._laenge < max))
//        {
//          max = obj._laenge;
//          maxCode = code;
//        }
//      }
//    }
//        
//    if (laenge < 0)
//    {
//      return codeOhneLaenge;
//    }
//    else if (min <= max)
//    {
//      double m = (max + min) / 2;
//      if (laenge > m)
//      {
//        return maxCode;
//      }
//      else
//      {
//        return minCode != null ? minCode : codeOhneLaenge;
//      }
//    }
//    else
//    {
//      // laenge ist grsser als die grsste versorgte Lnge
//      // min/minCode zeigen auf die grsste versorgte Lnge
//      return minCode != null ? minCode : codeOhneLaenge;
//      // Alternativ:
//      // nur return codeOhneLaenge;
//    }
//  }
  
  public static String getCodeZuLaenge(RDSEreignis ereignis, int laenge, Collection<String> zulaessigeCodes, boolean baustelle)
  {
    Map<String, RDSBeschreibung> resultMap = baustelle ? 
      DATEN_BAUSTELLE.get(ereignis) : DATEN_NORMAL.get(ereignis);
      
    for (String schluessel: resultMap.keySet())
    {
      RDSBeschreibung beschreibung = resultMap.get(schluessel);
      if (beschreibung._laenge == laenge)
      {
        return (zulaessigeCodes == null || zulaessigeCodes.contains(schluessel)) ? schluessel : null;
      }
    }
    
    return null;
  }
  
  public static String getTextZuCode(RDSEreignis ereignis, String code, boolean baustelle)
  {
    Map<String, RDSBeschreibung> resultMap = baustelle ? 
        DATEN_BAUSTELLE.get(ereignis) : DATEN_NORMAL.get(ereignis);
    
    RDSBeschreibung obj = resultMap.get(code);
    return obj != null ? obj._text : null;
  }
  
  public static RDSEreignis getRDSEreignisZuCode(String code)
  {
    Map[] maps = {DATEN_NORMAL, DATEN_BAUSTELLE};
    
    for (Map map: maps)
    {  
      Map<RDSEreignis, Map<String, RDSBeschreibung>> outerMap = (Map<RDSEreignis, Map<String, RDSBeschreibung>>)map;
      for (RDSEreignis rdsEreignis: outerMap.keySet())
      {
        Map<String, RDSBeschreibung> innerMap = outerMap.get(rdsEreignis);
        if (innerMap.keySet().contains(code))
        {
          return rdsEreignis;
        }
      }
    }
    
    return null;
  }
  
  private int _laenge;
  private String _text;
    
  private RDSBeschreibung(int laenge, String text)
  {
    _laenge = laenge;
    _text = text;
  }
 
}